/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "defaultsqldataadapter.h"

#include "datacontainer.h"
#include "refcontainer.h"
#include <QSqlRecord>
#include <QSqlTableModel>


namespace GerQ {

  DefaultSqlDataAdapter::DefaultSqlDataAdapter(
    const QString &aTableName, const QString &aIDColumn,
    void (*aRead)(DataObject*, SqlDataAdapter*),
    void (*aWrite)(DataObject*, SqlDataAdapter*),
    QString aConnection
  ): SqlDataAdapter(aTableName, aIDColumn, aConnection) {
    fRead = aRead;
    fWrite = aWrite;
  } // DefaultSqlDataAdapter::DefaultSqlDataAdapter

  void DefaultSqlDataAdapter::doRead(
    DataObject *aData, SqlDataAdapter *aSql
  ) {
    (*fRead)(aData, aSql);
  } // DefaultSqlDataAdapter::doRead

  void DefaultSqlDataAdapter::doWrite(
    DataObject *aData, SqlDataAdapter *aSql
  ) {
    (*fWrite)(aData, aSql);
  } // DefaultSqlDataAdapter::doWrite

  void DefaultSqlDataAdapter::newData(QVariant &aID) {
    QSqlTableModel *table = getTable();
    DataContainer *container = getContainer();
    QString IDColumn = getIDColumn();
    /*
      Clear the filter, so the new record won't be "filtered away" by accident:
    */
    table -> setFilter("");
    table -> select();
    int m = table -> columnCount();
    /*
      Workaround to the problem that QSqlTableModel only reads 256 records at
      once: Make sure that all data is read.
    */
    while (table -> canFetchMore()) {
      table -> fetchMore();
    } // while
    int n = table -> rowCount();
    table -> insertRows(n, 1);
    QSqlRecord record = table -> record(n);
    int iIDColumn = table -> fieldIndex(getIDColumn());
    /*
      Some databases (esp. SqLite) require every field of a newly inserted
      record to be initialized, or else they cannot be modified or deleted
      afterwards:
    */
    QString nullstr = container -> getNullID().toString();
    for (int i = 0; i < m; i++) {
      if (i != iIDColumn) {
        table -> setData(table -> index(n, i), nullstr);
      } // if
    } // for
    /*
      Finally we need to get a valid ID for the new record:
    */
    RefContainer *ctrParent = container -> getParent();
    if (ctrParent) {
      /*
        The DefaultSqlDataAdapter assumes that the ID field of the table for
	every derived class is meant to contain the same value as the ID field
	for the respective record in the table for the anchestor class. So let
	first the anchestor's data adapter create a new entry, then take the ID
	returned for the record created:
      */
      DataAdapter *adpParent = ctrParent -> getAdapter();
      adpParent -> newData(aID);
      table -> setData(table -> index(n, iIDColumn), aID);
      table -> submitAll();
    } else {
      /*
	If there aren't any parent containers, assume your table is the
	top-level table of the inheritance hierarchy, and it has an auto-inc ID
	field. Just submit the new record and take whatever ID you get back
	(since the SqlTableModel is sorted by ID in ascending order, the newly
	created record should still be the one at position n after the
	submitAll()):
      */
      for (int column = 0; column < table -> columnCount(); column++) {
        if (column != iIDColumn) {
	  table -> setData(table -> index(n, column), QVariant(0));
	} // if
      } // for
      table -> submitAll();
      record = table -> record(n);
      aID = record.value(IDColumn);
    } // if
  } // DefaultSqlDataAdapter::newData

} // GerQ
