/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GERQ_SORTEDLIST_H
#define GERQ_SORTEDLIST_H


#include <QList>


namespace GerQ {


  template <class TYPE> class SortedList {

    public:

      SortedList(int aComparator(TYPE, TYPE)) {
        fComparator = aComparator;
	fCompArgs = NULL;
	fArgs = NULL;
	fReverse = false;
      } // SortedList

      SortedList(int aCompArgs(TYPE, TYPE, void*), void *aArgs) {
        fComparator = NULL;
        fCompArgs = aCompArgs;
	fArgs = aArgs;
	fReverse = false;
      } // SortedList

      int insert(TYPE aData) {
        int i = getIndex(aData);
	if (i >= fData.count() || fData.at(i) != aData) {
	  fData.insert(i, aData);
	} // if
	return i;
      } // insert

      void remove(TYPE aData) {
	typename QList<TYPE>::iterator iter = fData.begin();
	while (*iter == aData && fData.count() > 0) {
	  fData.removeFirst();
	  iter = fData.begin();
	} // if
	iter++;
	while (iter != fData.end() && fData.count() > 0) {
	  TYPE data = *iter;
	  if (data == aData) {
	    fData.erase(iter);
	  } else {
	    iter++;
	  } // if
	} // while
      } // remove

      void removeAt(int aIndex) {
        fData.removeAt(aIndex);
      } // removeAt

      void clear() {
        while (fData.count() > 0) {
	  fData.removeFirst();
	} // while
      } // clear

      int getCount() {
        return fData.count();
      } // getCount

      TYPE getAt(int aIndex) {
        return fData.at(aIndex);
      } // getAt

      int getIndex(TYPE aData) {
        return findIndex(aData);
      } // getIndex

      bool getReverse() {
        return fReverse;
      } // getReverse

      void setReverse(bool aValue) {
        fReverse = aValue;
	resort();
      } // setReverse

      int getCompared(TYPE aData1, TYPE aData2) {
        int result = 0;
        if (fComparator) {
          result = (*fComparator)(aData1, aData2);
	} else if (fCompArgs) {
	  result = (*fCompArgs)(aData1, aData2, fArgs);
	} // if
	if (fReverse) {
	  result = -result;
	} // if
	return result;
      } // getCompared

      void *getArgs() {
        return fArgs;
      } // getArgs

      void setArgs(void *aValue) {
        fArgs = aValue;
      } // setArgs

      void resort() {
        QList<TYPE> data(fData);
	clear();
	typename QList<TYPE>::iterator iter = data.begin();
	while (iter != data.end()) {
	  insert(*iter);
	  iter++;
	} // while
      } // resort

      TYPE operator[](int aIndex) {
        return getAt(aIndex);
      } // []

    private:

      QList<TYPE> fData;
      int (*fComparator)(TYPE, TYPE);
      int (*fCompArgs)(TYPE, TYPE, void*);
      void *fArgs;
      bool fReverse;

      int findIndex(TYPE aData, int aLow = 0, int aHigh = -1) {
        int l = aLow;
	int h = aHigh;
	if (h < 0) {
	  int n = fData.size();
	  if (n == 0) {
	    return 0;
	  } else {
	    h = n - 1;
	  } // if
	} // if
	TYPE data = fData.at(l);
	if (getCompared(aData, data) <= 0) {
	  return l;
	} // if
	data = fData.at(h);
	int i = h + 1;
	if (getCompared(aData, data) > 0) {
	  return i;
	} // if
	if (h == l + 1) {
	  return h;
	} // if
	i = (l + h) / 2;
	data = fData.at(i);
	if (getCompared(aData, data) > 0) {
	  return findIndex(aData, i, h);
	} else {
	  return findIndex(aData, l, i);
	} // if
      } // findIndex

  }; // SortedList


} // GerQ


#endif
