/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "sqldataadapter.h"

#include "dataobject.h"
#include "refcontainer.h"
#include <QSqlRecord>


namespace GerQ {

  const char* SqlDataAdapter::FMT_DATE		= "yyyy-MM-dd";
  const char* SqlDataAdapter::FMT_TIME		= "hh:mm:ss.z";
  const char* SqlDataAdapter::FMT_DATETIME	= "yyyy-MM-dd hh:mm:ss.z";

  SqlDataAdapter::SqlDataAdapter(
    const QString &aTableName, const QString &aIDColumn, QString aConnection
  ) {
    fConnection = aConnection;
    if (aConnection == "") {
      fTable = new QSqlTableModel();
    } else {
      QSqlDatabase db = QSqlDatabase::database(aConnection);
      fTable = new QSqlTableModel(NULL, db);
    } // if
    fTable -> setTable(aTableName);
    fIDColumn = aIDColumn;
    int i = fTable -> fieldIndex(fIDColumn);
    fTable -> setSort(i, Qt::AscendingOrder);
    fTable -> select();
  } // SqlDataAdapter::SqlDataAdapter

  SqlDataAdapter::~SqlDataAdapter() {
    delete fTable;
  } // SqlDataAdapter::~SqlDataAdapter

  void SqlDataAdapter::erase(DataObject *aData) {
    gotoRecord(aData);
    if (fTable -> rowCount() == 1) {
      fTable -> removeRows(0, 1);
      fTable -> submitAll();
    } // if
  } // SqlDataAdapter::erase

  QString SqlDataAdapter::getConnection() {
    return fConnection;
  } // SqlDataAdapter::getConnection

  bool SqlDataAdapter::getFdBool(const QString &aName) {
    return getField(aName).toInt() == 1;
  } // SqlDataAdapter::getFdBool

  QDate SqlDataAdapter::getFdDate(const QString &aName) {
    QDate result = QDate();
    QString s = getField(aName).toString();
    if (!s.isEmpty()) {
      result = QDate::fromString(s, FMT_DATE);
    } // if
    return result;
  } // SqlDataAdapter::getFdDate

  QDateTime SqlDataAdapter::getFdDateTime(const QString &aName) {
    QDateTime result = QDateTime();
    QString s = getField(aName).toString();
    if (!s.isEmpty()) {
      result = QDateTime::fromString(s, FMT_DATETIME);
    } // if
    return result;
  } // SqlDataAdapter::getFdDateTime

  QTime SqlDataAdapter::getFdTime(const QString &aName) {
    QTime result = QTime();
    QString s = getField(aName).toString();
    if (!s.isEmpty()) {
      result = QTime::fromString(s, FMT_TIME);
    } // if
    return result;
  } // SqlDataAdapter::getFdTime

  QVariant SqlDataAdapter::getField(const QString &aName) {
    if (fTable -> rowCount() == 1) {
      QSqlRecord record = fTable -> record(0);
      return record.value(aName);
    } else {
      return QVariant();
    } // if
  } // SqlDataAdapter::getField

  QString SqlDataAdapter::getIDColumn() {
    return fIDColumn;
  } // SqlDataAdapter::getIDColumn

  QSqlTableModel* SqlDataAdapter::getTable() {
    return fTable;
  } // SqlDataAdapter::getTable

  QString SqlDataAdapter::getTableName() {
    return fTable -> tableName();
  } // SqlDataAdapter::getTableName

  void SqlDataAdapter::gotoRecord(DataObject *aData) {
    /* To be honest, this is rather a filter function than a "goto": */
    QString filter;
    QVariant ID = aData -> getID();
    switch (ID.type()) {
      case QVariant::Char:
      case QVariant::String:
        filter = QString().sprintf("%s = '%s'",
	  qPrintable(fIDColumn), qPrintable(ID.toString()));
	break;
      default:
        filter = QString().sprintf("%s = %s",
	  qPrintable(fIDColumn), qPrintable(ID.toString()));
    } // switch
    fTable -> setFilter(filter);
  } // SqlDataAdapter::gotoRecord

  bool SqlDataAdapter::read(DataObject *aData) {
    gotoRecord(aData);
    if (fTable -> rowCount() == 1) {
      doRead(aData, this);
      return true;
    } else {
      return false;
    } // if
  } // SqlDataAdapter::read

  void SqlDataAdapter::readAllIDs(QList<QVariant> &aIDs) {
    fTable -> setFilter("");
    /*
      Workaround to the problem that QSqlTableModel only reads 256 records at
      once: Make sure that all data is read.
    */
    while (fTable -> canFetchMore()) {
      fTable -> fetchMore();
    } // while
    int n = fTable -> rowCount();
    for (int i = 0; i < n; i++) {
      QSqlRecord record = fTable -> record(i);
      QVariant ID = record.value(fIDColumn);
      aIDs.append(ID);
    } // for
  } // SqlDataAdapter::readAll

  void SqlDataAdapter::reload() {
    fTable -> setFilter("");
    fTable -> select();
  } // SqlDataAdapter::reload

  void SqlDataAdapter::setFdBool(const QString &aName, bool aValue) {
    if (aValue) {
      setField(aName, 1);
    } else {
      setField(aName, 0);
    } // if
  } // SqlDataAdapter::setFdBool

  void SqlDataAdapter::setFdDate(const QString &aName, const QDate &aValue) {
    QString s = "";
    if (aValue.isValid()) {
      s = aValue.toString(FMT_DATE);
    } // if
    setField(aName, s);
  } // SqlDataAdapter::setFdDate

  void SqlDataAdapter::setFdDateTime(
    const QString &aName, const QDateTime &aValue
  ) {
    QString s = "";
    if (aValue.isValid()) {
      s = aValue.toString(FMT_DATETIME);
    } // if
    setField(aName, s);
  } // SqlDataAdapter::setFdDateTime

  void SqlDataAdapter::setFdTime(const QString &aName, const QTime &aValue) {
    QString s = "";
    if (aValue.isValid()) {
      s = aValue.toString(FMT_TIME);
    } // if
    setField(aName, s);
  } // SqlDataAdapter::setFdTime

  void SqlDataAdapter::setField(const QString &aName, const QVariant &aValue) {
    if (fTable -> rowCount() == 1) {
      int i = fTable -> fieldIndex(aName);
      fTable -> setData(fTable -> index(0, i), aValue);
    } // if
  } // SqlDataAdapter::setField

  void SqlDataAdapter::write(DataObject *aData) {
    gotoRecord(aData);
    if (fTable -> rowCount() == 1) {
      doWrite(aData, this);
      fTable -> submitAll();
    } // if
  } // SqlDataAdapter::write

} // GerQ
